@@ -63,7 +63,7 @@ BaseEstimator.__repr__ for pretty-printing estimators"""
 # - long sequences (lists, tuples, dict items) with more than N elements are
 #   shortened using ellipsis (', ...') at the end.
 
-from inspect import signature
+import inspect
 import pprint
 from collections import OrderedDict
 
@@ -89,16 +89,27 @@ def _changed_params(estimator):
     estimator with non-default values."""
 
     params = estimator.get_params(deep=False)
-    filtered_params = {}
     init_func = getattr(estimator.__init__, 'deprecated_original',
                         estimator.__init__)
-    init_params = signature(init_func).parameters
+    init_params = inspect.signature(init_func).parameters
     init_params = {name: param.default for name, param in init_params.items()}
-    for k, v in params.items():
+
+    def has_changed(k, v):
+        if k not in init_params:  # happens if k is part of a **kwargs
+            return True
+        if init_params[k] == inspect._empty:  # k has no default value
+            return True
+        # try to avoid calling repr on nested estimators
+        if (isinstance(v, BaseEstimator) and
+           v.__class__ != init_params[k].__class__):
+            return True
+        # Use repr as a last resort. It may be expensive.
         if (repr(v) != repr(init_params[k]) and
-                not (is_scalar_nan(init_params[k]) and is_scalar_nan(v))):
-            filtered_params[k] = v
-    return filtered_params
+           not (is_scalar_nan(init_params[k]) and is_scalar_nan(v))):
+            return True
+        return False
+
+    return {k: v for k, v in params.items() if has_changed(k, v)}
 
 
 class _EstimatorPrettyPrinter(pprint.PrettyPrinter):
