@@ -2,6 +2,7 @@
 
 import warnings
 import os
+import re
 
 from tempfile import NamedTemporaryFile
 from itertools import product
@@ -12,26 +13,23 @@ from pytest import importorskip
 import numpy as np
 import scipy.sparse as sp
 
-from sklearn.utils._testing import assert_raises
-from sklearn.utils._testing import assert_raises_regex
 from sklearn.utils._testing import assert_no_warnings
-from sklearn.utils._testing import assert_warns
-from sklearn.utils._testing import assert_warns_message
 from sklearn.utils._testing import ignore_warnings
 from sklearn.utils._testing import SkipTest
 from sklearn.utils._testing import assert_array_equal
 from sklearn.utils._testing import assert_allclose_dense_sparse
 from sklearn.utils._testing import assert_allclose
+from sklearn.utils._testing import _convert_container
 from sklearn.utils import as_float_array, check_array, check_symmetric
 from sklearn.utils import check_X_y
 from sklearn.utils import deprecated
 from sklearn.utils._mocking import MockDataFrame
+from sklearn.utils.fixes import np_version, parse_version
 from sklearn.utils.estimator_checks import _NotAnArray
 from sklearn.random_projection import _sparse_random_matrix
 from sklearn.linear_model import ARDRegression
 from sklearn.neighbors import KNeighborsClassifier
 from sklearn.ensemble import RandomForestRegressor
-from sklearn.exceptions import DataConversionWarning
 from sklearn.svm import SVR
 from sklearn.datasets import make_blobs
 from sklearn.utils import _safe_indexing
@@ -42,6 +40,7 @@ from sklearn.utils.validation import (
     assert_all_finite,
     check_memory,
     check_non_negative,
+    _num_features,
     _num_samples,
     check_scalar,
     _check_psd_eigenvalues,
@@ -50,15 +49,17 @@ from sklearn.utils.validation import (
     _allclose_dense_sparse,
     FLOAT_DTYPES)
 from sklearn.utils.validation import _check_fit_params
+from sklearn.utils.fixes import parse_version
 
 import sklearn
 
 from sklearn.exceptions import NotFittedError, PositiveSpectrumWarning
 
-from sklearn.utils._testing import assert_raise_message
 from sklearn.utils._testing import TempMemmap
 
 
+@pytest.mark.filterwarnings(
+    "ignore:the matrix subclass:PendingDeprecationWarning")
 def test_as_float_array():
     # Test function for as_float_array
     X = np.ones((3, 10), dtype=np.int32)
@@ -69,10 +70,10 @@ def test_as_float_array():
     X = X.astype(np.int64)
     X2 = as_float_array(X, copy=True)
     # Checking that the array wasn't overwritten
-    assert as_float_array(X, False) is not X
+    assert as_float_array(X, copy=False) is not X
     assert X2.dtype == np.float64
     # Test int dtypes <= 32bit
-    tested_dtypes = [np.bool,
+    tested_dtypes = [bool,
                      np.int8, np.int16, np.int32,
                      np.uint8, np.uint16, np.uint32]
     for dtype in tested_dtypes:
@@ -115,6 +116,8 @@ def test_as_float_array_nan(X):
     assert_allclose_dense_sparse(X_converted, X)
 
 
+@pytest.mark.filterwarnings(
+    "ignore:the matrix subclass:PendingDeprecationWarning")
 def test_np_matrix():
     # Confirm that input validation code does not return np.matrix
     X = np.arange(12).reshape(3, 4)
@@ -168,7 +171,7 @@ def test_ordering():
     [np.asarray, sp.csr_matrix]
 )
 def test_check_array_force_all_finite_valid(value, force_all_finite, retype):
-    X = retype(np.arange(4).reshape(2, 2).astype(np.float))
+    X = retype(np.arange(4).reshape(2, 2).astype(float))
     X[0, 0] = value
     X_checked = check_array(X, force_all_finite=force_all_finite,
                             accept_sparse=True)
@@ -189,7 +192,7 @@ def test_check_array_force_all_finite_valid(value, force_all_finite, retype):
 )
 def test_check_array_force_all_finiteinvalid(value, force_all_finite,
                                              match_msg, retype):
-    X = retype(np.arange(4).reshape(2, 2).astype(np.float))
+    X = retype(np.arange(4).reshape(2, 2).astype(float))
     X[0, 0] = value
     with pytest.raises(ValueError, match=match_msg):
         check_array(X, force_all_finite=force_all_finite,
@@ -217,7 +220,7 @@ def test_check_array_force_all_finite_object():
       "Input contains NaN, infinity or a value too large for.*int"),
      (np.array([[1, np.inf]]),
       "Input contains NaN, infinity or a value too large for.*int"),
-     (np.array([[1, np.nan]], dtype=np.object),
+     (np.array([[1, np.nan]], dtype=object),
       "cannot convert float NaN to integer")]
 )
 @pytest.mark.parametrize("force_all_finite", [True, False])
@@ -226,7 +229,7 @@ def test_check_array_force_all_finite_object_unsafe_casting(
     # casting a float array containing NaN or inf to int dtype should
     # raise an error irrespective of the force_all_finite parameter.
     with pytest.raises(ValueError, match=err_msg):
-        check_array(X, dtype=np.int, force_all_finite=force_all_finite)
+        check_array(X, dtype=int, force_all_finite=force_all_finite)
 
 
 @ignore_warnings
@@ -235,29 +238,35 @@ def test_check_array():
     # raise error on sparse inputs
     X = [[1, 2], [3, 4]]
     X_csr = sp.csr_matrix(X)
-    assert_raises(TypeError, check_array, X_csr)
+    with pytest.raises(TypeError):
+        check_array(X_csr)
+
     # ensure_2d=False
     X_array = check_array([0, 1, 2], ensure_2d=False)
     assert X_array.ndim == 1
     # ensure_2d=True with 1d array
-    assert_raise_message(ValueError, 'Expected 2D array, got 1D array instead',
-                         check_array, [0, 1, 2], ensure_2d=True)
+    with pytest.raises(ValueError, match="Expected 2D array,"
+                                         " got 1D array instead"):
+        check_array([0, 1, 2], ensure_2d=True)
+
     # ensure_2d=True with scalar array
-    assert_raise_message(ValueError,
-                         'Expected 2D array, got scalar array instead',
-                         check_array, 10, ensure_2d=True)
+    with pytest.raises(ValueError, match="Expected 2D array,"
+                                         " got scalar array instead"):
+        check_array(10, ensure_2d=True)
+
     # don't allow ndim > 3
     X_ndim = np.arange(8).reshape(2, 2, 2)
-    assert_raises(ValueError, check_array, X_ndim)
+    with pytest.raises(ValueError):
+        check_array(X_ndim)
     check_array(X_ndim, allow_nd=True)  # doesn't raise
 
     # dtype and order enforcement.
     X_C = np.arange(4).reshape(2, 2).copy("C")
     X_F = X_C.copy("F")
-    X_int = X_C.astype(np.int)
-    X_float = X_C.astype(np.float)
+    X_int = X_C.astype(int)
+    X_float = X_C.astype(float)
     Xs = [X_C, X_F, X_int, X_float]
-    dtypes = [np.int32, np.int, np.float, np.float32, None, np.bool, object]
+    dtypes = [np.int32, int, float, np.float32, None, bool, object]
     orders = ['C', 'F', None]
     copys = [True, False]
 
@@ -286,8 +295,8 @@ def test_check_array():
     X_csc = sp.csc_matrix(X_C)
     X_coo = X_csc.tocoo()
     X_dok = X_csc.todok()
-    X_int = X_csc.astype(np.int)
-    X_float = X_csc.astype(np.float)
+    X_int = X_csc.astype(int)
+    X_float = X_csc.astype(float)
 
     Xs = [X_csc, X_coo, X_dok, X_int, X_float]
     accept_sparses = [['csr', 'coo'], ['coo', 'dok']]
@@ -297,6 +306,7 @@ def test_check_array():
             X_checked = check_array(X, dtype=dtype,
                                     accept_sparse=accept_sparse, copy=copy)
         if (dtype is object or sp.isspmatrix_dok(X)) and len(w):
+            # XXX unreached code as of v0.22
             message = str(w[0].message)
             messages = ["object dtype is not supported by sparse matrices",
                         "Can't check dok sparse matrix for nan or inf."]
@@ -325,37 +335,97 @@ def test_check_array():
     X_dense = check_array([[1, 2], [3, 4]])
     assert isinstance(X_dense, np.ndarray)
     # raise on too deep lists
-    assert_raises(ValueError, check_array, X_ndim.tolist())
+    with pytest.raises(ValueError):
+        check_array(X_ndim.tolist())
     check_array(X_ndim.tolist(), allow_nd=True)  # doesn't raise
+
     # convert weird stuff to arrays
     X_no_array = _NotAnArray(X_dense)
     result = check_array(X_no_array)
     assert isinstance(result, np.ndarray)
 
-    # deprecation warning if string-like array with dtype="numeric"
-    expected_warn_regex = r"converted to decimal numbers if dtype='numeric'"
-    X_str = [['11', '12'], ['13', 'xx']]
-    for X in [X_str, np.array(X_str, dtype='U'), np.array(X_str, dtype='S')]:
-        with pytest.warns(FutureWarning, match=expected_warn_regex):
-            check_array(X, dtype="numeric")
 
-    # deprecation warning if byte-like array with dtype="numeric"
-    X_bytes = [[b'a', b'b'], [b'c', b'd']]
-    for X in [X_bytes, np.array(X_bytes, dtype='V1')]:
-        with pytest.warns(FutureWarning, match=expected_warn_regex):
-            check_array(X, dtype="numeric")
+# TODO: Check for error in 1.1 when implicit conversation is removed
+@pytest.mark.parametrize("X", [
+   [['1', '2'], ['3', '4']],
+   np.array([['1', '2'], ['3', '4']], dtype='U'),
+   np.array([['1', '2'], ['3', '4']], dtype='S'),
+   [[b'1', b'2'], [b'3', b'4']],
+   np.array([[b'1', b'2'], [b'3', b'4']], dtype='V1')
+])
+def test_check_array_numeric_warns(X):
+    """Test that check_array warns when it converts a bytes/string into a
+    float."""
+    expected_msg = (r"Arrays of bytes/strings is being converted to decimal .*"
+                    r"deprecated in 0.24 and will be removed in 1.1")
+    with pytest.warns(FutureWarning, match=expected_msg):
+        check_array(X, dtype="numeric")
 
 
+# TODO: remove in 1.1
+@ignore_warnings(category=FutureWarning)
+@pytest.mark.parametrize("X", [
+   [['11', '12'], ['13', 'xx']],
+   np.array([['11', '12'], ['13', 'xx']], dtype='U'),
+   np.array([['11', '12'], ['13', 'xx']], dtype='S'),
+   [[b'a', b'b'], [b'c', b'd']],
+   np.array([[b'a', b'b'], [b'c', b'd']], dtype='V1')
+])
+def test_check_array_dtype_numeric_errors(X):
+    """Error when string-ike array can not be converted"""
+    if (np_version < parse_version("1.14")
+            and hasattr(X, "dtype") and X.dtype.kind == "V"):
+        pytest.skip("old numpy would convert V dtype into float silently")
+    expected_warn_msg = "Unable to convert array of bytes/strings"
+    with pytest.raises(ValueError, match=expected_warn_msg):
+        check_array(X, dtype="numeric")
+
+
+@pytest.mark.parametrize("pd_dtype", ["Int8", "Int16", "UInt8", "UInt16"])
+@pytest.mark.parametrize("dtype, expected_dtype", [
+    ([np.float32, np.float64], np.float32),
+    (np.float64, np.float64),
+    ("numeric", np.float64),
+])
+def test_check_array_pandas_na_support(pd_dtype, dtype, expected_dtype):
+    # Test pandas IntegerArray with pd.NA
+    pd = pytest.importorskip('pandas', minversion="1.0")
+
+    X_np = np.array([[1, 2, 3, np.nan, np.nan],
+                     [np.nan, np.nan, 8, 4, 6],
+                     [1, 2, 3, 4, 5]]).T
+
+    # Creates dataframe with IntegerArrays with pd.NA
+    X = pd.DataFrame(X_np, dtype=pd_dtype, columns=['a', 'b', 'c'])
+    # column c has no nans
+    X['c'] = X['c'].astype('float')
+    X_checked = check_array(X, force_all_finite='allow-nan', dtype=dtype)
+    assert_allclose(X_checked, X_np)
+    assert X_checked.dtype == expected_dtype
+
+    X_checked = check_array(X, force_all_finite=False, dtype=dtype)
+    assert_allclose(X_checked, X_np)
+    assert X_checked.dtype == expected_dtype
+
+    msg = "Input contains NaN, infinity"
+    with pytest.raises(ValueError, match=msg):
+        check_array(X, force_all_finite=True)
+
+
+# TODO: remove test in 1.1 once this behavior is deprecated
 def test_check_array_pandas_dtype_object_conversion():
     # test that data-frame like objects with dtype object
     # get converted
-    X = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]], dtype=np.object)
+    X = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]], dtype=object)
     X_df = MockDataFrame(X)
-    assert check_array(X_df).dtype.kind == "f"
-    assert check_array(X_df, ensure_2d=False).dtype.kind == "f"
+    with pytest.warns(FutureWarning):
+        assert check_array(X_df).dtype.kind == "f"
+    with pytest.warns(FutureWarning):
+        assert check_array(X_df, ensure_2d=False).dtype.kind == "f"
     # smoke-test against dataframes with column named "dtype"
     X_df.dtype = "Hans"
-    assert check_array(X_df, ensure_2d=False).dtype.kind == "f"
+    with pytest.warns(FutureWarning):
+        assert check_array(X_df, ensure_2d=False).dtype.kind == "f"
 
 
 def test_check_array_pandas_dtype_casting():
@@ -415,55 +485,18 @@ def test_check_array_dtype_stability():
 
 def test_check_array_dtype_warning():
     X_int_list = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
-    X_float64 = np.asarray(X_int_list, dtype=np.float64)
     X_float32 = np.asarray(X_int_list, dtype=np.float32)
     X_int64 = np.asarray(X_int_list, dtype=np.int64)
-    X_csr_float64 = sp.csr_matrix(X_float64)
     X_csr_float32 = sp.csr_matrix(X_float32)
     X_csc_float32 = sp.csc_matrix(X_float32)
     X_csc_int32 = sp.csc_matrix(X_int64, dtype=np.int32)
-    y = [0, 0, 1]
     integer_data = [X_int64, X_csc_int32]
-    float64_data = [X_float64, X_csr_float64]
     float32_data = [X_float32, X_csr_float32, X_csc_float32]
     for X in integer_data:
         X_checked = assert_no_warnings(check_array, X, dtype=np.float64,
                                        accept_sparse=True)
         assert X_checked.dtype == np.float64
 
-        X_checked = assert_warns(DataConversionWarning, check_array, X,
-                                 dtype=np.float64,
-                                 accept_sparse=True, warn_on_dtype=True)
-        assert X_checked.dtype == np.float64
-
-        # Check that the warning message includes the name of the Estimator
-        X_checked = assert_warns_message(DataConversionWarning,
-                                         'SomeEstimator',
-                                         check_array, X,
-                                         dtype=[np.float64, np.float32],
-                                         accept_sparse=True,
-                                         warn_on_dtype=True,
-                                         estimator='SomeEstimator')
-        assert X_checked.dtype == np.float64
-
-        X_checked, y_checked = assert_warns_message(
-            DataConversionWarning, 'KNeighborsClassifier',
-            check_X_y, X, y, dtype=np.float64, accept_sparse=True,
-            warn_on_dtype=True, estimator=KNeighborsClassifier())
-
-        assert X_checked.dtype == np.float64
-
-    for X in float64_data:
-        with pytest.warns(None) as record:
-            warnings.simplefilter("ignore", FutureWarning)  # 0.23
-            X_checked = check_array(X, dtype=np.float64,
-                                    accept_sparse=True, warn_on_dtype=True)
-            assert X_checked.dtype == np.float64
-            X_checked = check_array(X, dtype=np.float64,
-                                    accept_sparse=True, warn_on_dtype=False)
-            assert X_checked.dtype == np.float64
-        assert len(record) == 0
-
     for X in float32_data:
         X_checked = assert_no_warnings(check_array, X,
                                        dtype=[np.float64, np.float32],
@@ -487,41 +520,29 @@ def test_check_array_dtype_warning():
     assert X_checked.format == 'csr'
 
 
-def test_check_array_warn_on_dtype_deprecation():
-    X = np.asarray([[0.0], [1.0]])
-    Y = np.asarray([[2.0], [3.0]])
-    with pytest.warns(FutureWarning,
-                      match="'warn_on_dtype' is deprecated"):
-        check_array(X, warn_on_dtype=True)
-    with pytest.warns(FutureWarning,
-                      match="'warn_on_dtype' is deprecated"):
-        check_X_y(X, Y, warn_on_dtype=True)
-
-
 def test_check_array_accept_sparse_type_exception():
     X = [[1, 2], [3, 4]]
     X_csr = sp.csr_matrix(X)
     invalid_type = SVR()
 
     msg = ("A sparse matrix was passed, but dense data is required. "
-           "Use X.toarray() to convert to a dense numpy array.")
-    assert_raise_message(TypeError, msg,
-                         check_array, X_csr, accept_sparse=False)
+           r"Use X.toarray\(\) to convert to a dense numpy array.")
+    with pytest.raises(TypeError, match=msg):
+        check_array(X_csr, accept_sparse=False)
 
     msg = ("Parameter 'accept_sparse' should be a string, "
-           "boolean or list of strings. You provided 'accept_sparse={}'.")
-    assert_raise_message(ValueError, msg.format(invalid_type),
-                         check_array, X_csr, accept_sparse=invalid_type)
+           "boolean or list of strings. You provided 'accept_sparse=.*'.")
+    with pytest.raises(ValueError, match=msg):
+        check_array(X_csr, accept_sparse=invalid_type)
 
     msg = ("When providing 'accept_sparse' as a tuple or list, "
            "it must contain at least one string value.")
-    assert_raise_message(ValueError, msg.format([]),
-                         check_array, X_csr, accept_sparse=[])
-    assert_raise_message(ValueError, msg.format(()),
-                         check_array, X_csr, accept_sparse=())
-
-    assert_raise_message(TypeError, "SVR",
-                         check_array, X_csr, accept_sparse=[invalid_type])
+    with pytest.raises(ValueError, match=msg):
+        check_array(X_csr, accept_sparse=[])
+    with pytest.raises(ValueError, match=msg):
+        check_array(X_csr, accept_sparse=())
+    with pytest.raises(TypeError, match="SVR"):
+        check_array(X_csr, accept_sparse=[invalid_type])
 
 
 def test_check_array_accept_sparse_no_exception():
@@ -552,57 +573,64 @@ def test_check_array_accept_large_sparse_raise_exception(X_64bit):
     # When large sparse are not allowed
     msg = ("Only sparse matrices with 32-bit integer indices "
            "are accepted. Got int64 indices.")
-    assert_raise_message(ValueError, msg,
-                         check_array, X_64bit,
-                         accept_sparse=True,
-                         accept_large_sparse=False)
+    with pytest.raises(ValueError, match=msg):
+        check_array(X_64bit, accept_sparse=True, accept_large_sparse=False)
 
 
 def test_check_array_min_samples_and_features_messages():
     # empty list is considered 2D by default:
-    msg = "0 feature(s) (shape=(1, 0)) while a minimum of 1 is required."
-    assert_raise_message(ValueError, msg, check_array, [[]])
+    msg = r"0 feature\(s\) \(shape=\(1, 0\)\) while a minimum of 1 is" \
+          " required."
+    with pytest.raises(ValueError, match=msg):
+        check_array([[]])
 
     # If considered a 1D collection when ensure_2d=False, then the minimum
     # number of samples will break:
-    msg = "0 sample(s) (shape=(0,)) while a minimum of 1 is required."
-    assert_raise_message(ValueError, msg, check_array, [], ensure_2d=False)
+    msg = r"0 sample\(s\) \(shape=\(0,\)\) while a minimum of 1 is required."
+    with pytest.raises(ValueError, match=msg):
+        check_array([], ensure_2d=False)
 
     # Invalid edge case when checking the default minimum sample of a scalar
-    msg = "Singleton array array(42) cannot be considered a valid collection."
-    assert_raise_message(TypeError, msg, check_array, 42, ensure_2d=False)
+    msg = r"Singleton array array\(42\) cannot be considered a valid" \
+          " collection."
+    with pytest.raises(TypeError, match=msg):
+        check_array(42, ensure_2d=False)
 
     # Simulate a model that would need at least 2 samples to be well defined
     X = np.ones((1, 10))
     y = np.ones(1)
-    msg = "1 sample(s) (shape=(1, 10)) while a minimum of 2 is required."
-    assert_raise_message(ValueError, msg, check_X_y, X, y,
-                         ensure_min_samples=2)
+    msg = r"1 sample\(s\) \(shape=\(1, 10\)\) while a minimum of 2 is" \
+          " required."
+    with pytest.raises(ValueError, match=msg):
+        check_X_y(X, y, ensure_min_samples=2)
 
     # The same message is raised if the data has 2 dimensions even if this is
     # not mandatory
-    assert_raise_message(ValueError, msg, check_X_y, X, y,
-                         ensure_min_samples=2, ensure_2d=False)
+    with pytest.raises(ValueError, match=msg):
+        check_X_y(X, y, ensure_min_samples=2, ensure_2d=False)
 
     # Simulate a model that would require at least 3 features (e.g. SelectKBest
     # with k=3)
     X = np.ones((10, 2))
     y = np.ones(2)
-    msg = "2 feature(s) (shape=(10, 2)) while a minimum of 3 is required."
-    assert_raise_message(ValueError, msg, check_X_y, X, y,
-                         ensure_min_features=3)
+    msg = r"2 feature\(s\) \(shape=\(10, 2\)\) while a minimum of 3 is" \
+          " required."
+    with pytest.raises(ValueError, match=msg):
+        check_X_y(X, y, ensure_min_features=3)
 
     # Only the feature check is enabled whenever the number of dimensions is 2
     # even if allow_nd is enabled:
-    assert_raise_message(ValueError, msg, check_X_y, X, y,
-                         ensure_min_features=3, allow_nd=True)
+    with pytest.raises(ValueError, match=msg):
+        check_X_y(X, y, ensure_min_features=3, allow_nd=True)
 
     # Simulate a case where a pipeline stage as trimmed all the features of a
     # 2D dataset.
     X = np.empty(0).reshape(10, 0)
     y = np.ones(10)
-    msg = "0 feature(s) (shape=(10, 0)) while a minimum of 1 is required."
-    assert_raise_message(ValueError, msg, check_X_y, X, y)
+    msg = r"0 feature\(s\) \(shape=\(10, 0\)\) while a minimum of 1 is" \
+          " required."
+    with pytest.raises(ValueError, match=msg):
+        check_X_y(X, y)
 
     # nd-data is not checked for any minimum number of features by default:
     X = np.ones((10, 0, 28, 28))
@@ -614,41 +642,41 @@ def test_check_array_min_samples_and_features_messages():
 
 def test_check_array_complex_data_error():
     X = np.array([[1 + 2j, 3 + 4j, 5 + 7j], [2 + 3j, 4 + 5j, 6 + 7j]])
-    assert_raises_regex(
-        ValueError, "Complex data not supported", check_array, X)
+    with pytest.raises(ValueError, match="Complex data not supported"):
+        check_array(X)
 
     # list of lists
     X = [[1 + 2j, 3 + 4j, 5 + 7j], [2 + 3j, 4 + 5j, 6 + 7j]]
-    assert_raises_regex(
-        ValueError, "Complex data not supported", check_array, X)
+    with pytest.raises(ValueError, match="Complex data not supported"):
+        check_array(X)
 
     # tuple of tuples
     X = ((1 + 2j, 3 + 4j, 5 + 7j), (2 + 3j, 4 + 5j, 6 + 7j))
-    assert_raises_regex(
-        ValueError, "Complex data not supported", check_array, X)
+    with pytest.raises(ValueError, match="Complex data not supported"):
+        check_array(X)
 
     # list of np arrays
     X = [np.array([1 + 2j, 3 + 4j, 5 + 7j]),
          np.array([2 + 3j, 4 + 5j, 6 + 7j])]
-    assert_raises_regex(
-        ValueError, "Complex data not supported", check_array, X)
+    with pytest.raises(ValueError, match="Complex data not supported"):
+        check_array(X)
 
     # tuple of np arrays
     X = (np.array([1 + 2j, 3 + 4j, 5 + 7j]),
          np.array([2 + 3j, 4 + 5j, 6 + 7j]))
-    assert_raises_regex(
-        ValueError, "Complex data not supported", check_array, X)
+    with pytest.raises(ValueError, match="Complex data not supported"):
+        check_array(X)
 
     # dataframe
     X = MockDataFrame(
         np.array([[1 + 2j, 3 + 4j, 5 + 7j], [2 + 3j, 4 + 5j, 6 + 7j]]))
-    assert_raises_regex(
-        ValueError, "Complex data not supported", check_array, X)
+    with pytest.raises(ValueError, match="Complex data not supported"):
+        check_array(X)
 
     # sparse matrix
     X = sp.coo_matrix([[0, 1 + 2j], [0, 0]])
-    assert_raises_regex(
-        ValueError, "Complex data not supported", check_array, X)
+    with pytest.raises(ValueError, match="Complex data not supported"):
+        check_array(X)
 
 
 def test_has_fit_parameter():
@@ -681,13 +709,16 @@ def test_check_symmetric():
                    'bsr': sp.bsr_matrix(arr_asym)}
 
     # check error for bad inputs
-    assert_raises(ValueError, check_symmetric, arr_bad)
+    with pytest.raises(ValueError):
+        check_symmetric(arr_bad)
 
     # check that asymmetric arrays are properly symmetrized
     for arr_format, arr in test_arrays.items():
         # Check for warnings and errors
-        assert_warns(UserWarning, check_symmetric, arr)
-        assert_raises(ValueError, check_symmetric, arr, raise_exception=True)
+        with pytest.warns(UserWarning):
+            check_symmetric(arr)
+        with pytest.raises(ValueError):
+            check_symmetric(arr, raise_exception=True)
 
         output = check_symmetric(arr, raise_warning=False)
         if sp.issparse(output):
@@ -699,15 +730,19 @@ def test_check_symmetric():
 
 def test_check_is_fitted():
     # Check is TypeError raised when non estimator instance passed
-    assert_raises(TypeError, check_is_fitted, ARDRegression)
-    assert_raises(TypeError, check_is_fitted, "SVR")
+    with pytest.raises(TypeError):
+        check_is_fitted(ARDRegression)
+    with pytest.raises(TypeError):
+        check_is_fitted("SVR")
 
     ard = ARDRegression()
     svr = SVR()
 
     try:
-        assert_raises(NotFittedError, check_is_fitted, ard)
-        assert_raises(NotFittedError, check_is_fitted, svr)
+        with pytest.raises(NotFittedError):
+            check_is_fitted(ard)
+        with pytest.raises(NotFittedError):
+            check_is_fitted(svr)
     except ValueError:
         assert False, "check_is_fitted failed with ValueError"
 
@@ -779,18 +814,19 @@ def test_check_consistent_length():
     check_consistent_length([1], [2], [3], [4], [5])
     check_consistent_length([[1, 2], [[1, 2]]], [1, 2], ['a', 'b'])
     check_consistent_length([1], (2,), np.array([3]), sp.csr_matrix((1, 2)))
-    assert_raises_regex(ValueError, 'inconsistent numbers of samples',
-                        check_consistent_length, [1, 2], [1])
-    assert_raises_regex(TypeError, r"got <\w+ 'int'>",
-                        check_consistent_length, [1, 2], 1)
-    assert_raises_regex(TypeError, r"got <\w+ 'object'>",
-                        check_consistent_length, [1, 2], object())
+    with pytest.raises(ValueError, match="inconsistent numbers of samples"):
+        check_consistent_length([1, 2], [1])
+    with pytest.raises(TypeError, match=r"got <\w+ 'int'>"):
+        check_consistent_length([1, 2], 1)
+    with pytest.raises(TypeError, match=r"got <\w+ 'object'>"):
+        check_consistent_length([1, 2], object())
+
+    with pytest.raises(TypeError):
+        check_consistent_length([1, 2], np.array(1))
 
-    assert_raises(TypeError, check_consistent_length, [1, 2], np.array(1))
     # Despite ensembles having __len__ they must raise TypeError
-    assert_raises_regex(TypeError, 'Expected sequence or array-like',
-                        check_consistent_length, [1, 2],
-                        RandomForestRegressor())
+    with pytest.raises(TypeError, match="Expected sequence or array-like"):
+        check_consistent_length([1, 2], RandomForestRegressor())
     # XXX: We should have a test with a string, but what is correct behaviour?
 
 
@@ -808,11 +844,13 @@ def test_check_dataframe_fit_attribute():
 
 def test_suppress_validation():
     X = np.array([0, np.inf])
-    assert_raises(ValueError, assert_all_finite, X)
+    with pytest.raises(ValueError):
+        assert_all_finite(X)
     sklearn.set_config(assume_finite=True)
     assert_all_finite(X)
     sklearn.set_config(assume_finite=False)
-    assert_raises(ValueError, assert_all_finite, X)
+    with pytest.raises(ValueError):
+        assert_all_finite(X)
 
 
 def test_check_array_series():
@@ -827,44 +865,6 @@ def test_check_array_series():
     assert_array_equal(res, np.array(['a', 'b', 'c'], dtype=object))
 
 
-def test_check_dataframe_warns_on_dtype():
-    # Check that warn_on_dtype also works for DataFrames.
-    # https://github.com/scikit-learn/scikit-learn/issues/10948
-    pd = importorskip("pandas")
-
-    df = pd.DataFrame([[1, 2, 3], [4, 5, 6]], dtype=object)
-    assert_warns_message(DataConversionWarning,
-                         "Data with input dtype object were all converted to "
-                         "float64.",
-                         check_array, df, dtype=np.float64, warn_on_dtype=True)
-    assert_warns(DataConversionWarning, check_array, df,
-                 dtype='numeric', warn_on_dtype=True)
-    with pytest.warns(None) as record:
-        warnings.simplefilter("ignore", FutureWarning)  # 0.23
-        check_array(df, dtype='object', warn_on_dtype=True)
-    assert len(record) == 0
-
-    # Also check that it raises a warning for mixed dtypes in a DataFrame.
-    df_mixed = pd.DataFrame([['1', 2, 3], ['4', 5, 6]])
-    assert_warns(DataConversionWarning, check_array, df_mixed,
-                 dtype=np.float64, warn_on_dtype=True)
-    assert_warns(DataConversionWarning, check_array, df_mixed,
-                 dtype='numeric', warn_on_dtype=True)
-    assert_warns(DataConversionWarning, check_array, df_mixed,
-                 dtype=object, warn_on_dtype=True)
-
-    # Even with numerical dtypes, a conversion can be made because dtypes are
-    # uniformized throughout the array.
-    df_mixed_numeric = pd.DataFrame([[1., 2, 3], [4., 5, 6]])
-    assert_warns(DataConversionWarning, check_array, df_mixed_numeric,
-                 dtype='numeric', warn_on_dtype=True)
-    with pytest.warns(None) as record:
-        warnings.simplefilter("ignore", FutureWarning)  # 0.23
-        check_array(df_mixed_numeric.astype(int),
-                    dtype='numeric', warn_on_dtype=True)
-    assert len(record) == 0
-
-
 def test_check_dataframe_mixed_float_dtypes():
     # pandas dataframe will coerce a boolean into a object, this is a mismatch
     # with np.result_type which will return a float
@@ -882,7 +882,7 @@ def test_check_dataframe_mixed_float_dtypes():
     expected_array = np.array(
         [[1.0, 0.0, 1.0],
          [2.0, 0.1, 0.0],
-         [3.0, 2.1, 1.0]], dtype=np.float)
+         [3.0, 2.1, 1.0]], dtype=float)
     assert_allclose_dense_sparse(array, expected_array)
 
 
@@ -904,14 +904,16 @@ def test_check_memory():
     dummy = DummyMemory()
     memory = check_memory(dummy)
     assert memory is dummy
-    assert_raises_regex(ValueError, "'memory' should be None, a string or"
-                        " have the same interface as joblib.Memory."
-                        " Got memory='1' instead.", check_memory, 1)
+
+    msg = "'memory' should be None, a string or have the same interface as" \
+          " joblib.Memory. Got memory='1' instead."
+    with pytest.raises(ValueError, match=msg):
+        check_memory(1)
     dummy = WrongDummyMemory()
-    assert_raises_regex(ValueError, "'memory' should be None, a string or"
-                        " have the same interface as joblib.Memory."
-                        " Got memory='{}' instead.".format(dummy),
-                        check_memory, dummy)
+    msg = "'memory' should be None, a string or have the same interface as" \
+          " joblib.Memory. Got memory='{}' instead.".format(dummy)
+    with pytest.raises(ValueError, match=msg):
+        check_memory(dummy)
 
 
 @pytest.mark.parametrize('copy', [True, False])
@@ -939,13 +941,15 @@ def test_check_non_negative(retype):
 
     A[0, 0] = -1
     X = retype(A)
-    assert_raises_regex(ValueError, "Negative ", check_non_negative, X, "")
+    with pytest.raises(ValueError, match="Negative "):
+        check_non_negative(X, "")
 
 
 def test_check_X_y_informative_error():
     X = np.ones((2, 2))
     y = None
-    assert_raise_message(ValueError, "y cannot be None", check_X_y, X, y)
+    with pytest.raises(ValueError, match="y cannot be None"):
+        check_X_y(X, y)
 
 
 def test_retrieve_samples_from_non_standard_shape():
@@ -975,7 +979,8 @@ def test_check_scalar_valid(x, target_type, min_val, max_val):
     """Test that check_scalar returns no error/warning if valid inputs are
     provided"""
     with pytest.warns(None) as record:
-        check_scalar(x, "test_name", target_type, min_val, max_val)
+        check_scalar(x, "test_name", target_type=target_type,
+                     min_val=min_val, max_val=max_val)
     assert len(record) == 0
 
 
@@ -1106,7 +1111,7 @@ def test_check_sample_weight():
     assert sample_weight.dtype == np.float32
 
     # int dtype will be converted to float64 instead
-    X = np.ones((5, 2), dtype=np.int)
+    X = np.ones((5, 2), dtype=int)
     sample_weight = _check_sample_weight(None, X, dtype=X.dtype)
     assert sample_weight.dtype == np.float64
 
@@ -1160,6 +1165,25 @@ def test_deprecate_positional_args_warns_for_function():
                       match=r"Pass b=2 as keyword args"):
         f2(1, 2)
 
+    # The * is place before a keyword only argument without a default value
+    @_deprecate_positional_args
+    def f3(a, *, b, c=1, d=1):
+        pass
+
+    with pytest.warns(FutureWarning,
+                      match=r"Pass b=2 as keyword args"):
+        f3(1, 2)
+
+
+def test_deprecate_positional_args_warns_for_function_version():
+    @_deprecate_positional_args(version="1.1")
+    def f1(a, *, b):
+        pass
+
+    with pytest.warns(FutureWarning,
+                      match=r"From version 1.1 passing these as positional"):
+        f1(1, 2)
+
 
 def test_deprecate_positional_args_warns_for_class():
 
@@ -1216,3 +1240,147 @@ def test_check_fit_params(indices):
         result['sparse-col'],
         _safe_indexing(fit_params['sparse-col'], indices_)
     )
+
+
+@pytest.mark.parametrize('sp_format', [True, 'csr', 'csc', 'coo', 'bsr'])
+def test_check_sparse_pandas_sp_format(sp_format):
+    # check_array converts pandas dataframe with only sparse arrays into
+    # sparse matrix
+    pd = pytest.importorskip("pandas", minversion="0.25.0")
+    sp_mat = _sparse_random_matrix(10, 3)
+
+    sdf = pd.DataFrame.sparse.from_spmatrix(sp_mat)
+    result = check_array(sdf, accept_sparse=sp_format)
+
+    if sp_format is True:
+        # by default pandas converts to coo when accept_sparse is True
+        sp_format = 'coo'
+
+    assert sp.issparse(result)
+    assert result.format == sp_format
+    assert_allclose_dense_sparse(sp_mat, result)
+
+
+@pytest.mark.parametrize(
+    "ntype1, ntype2",
+    [
+        ("longdouble", "float16"),
+        ("float16", "float32"),
+        ("float32", "double"),
+        ("int16", "int32"),
+        ("int32", "long"),
+        ("byte", "uint16"),
+        ("ushort", "uint32"),
+        ("uint32", "uint64"),
+        ("uint8", "int8"),
+    ]
+)
+def test_check_pandas_sparse_invalid(ntype1, ntype2):
+    """check that we raise an error with dataframe having
+    sparse extension arrays with unsupported mixed dtype
+    and pandas version below 1.1. pandas versions 1.1 and
+    above fixed this issue so no error will be raised."""
+    pd = pytest.importorskip("pandas", minversion="0.25.0")
+    df = pd.DataFrame({'col1': pd.arrays.SparseArray([0, 1, 0],
+                                                     dtype=ntype1),
+                       'col2': pd.arrays.SparseArray([1, 0, 1],
+                                                     dtype=ntype2)})
+
+    if parse_version(pd.__version__) < parse_version('1.1'):
+        err_msg = "Pandas DataFrame with mixed sparse extension arrays"
+        with pytest.raises(ValueError, match=err_msg):
+            check_array(df, accept_sparse=['csr', 'csc'])
+    else:
+        # pandas fixed this issue at 1.1 so from here on,
+        # no error will be raised.
+        check_array(df, accept_sparse=['csr', 'csc'])
+
+
+@pytest.mark.parametrize(
+    "ntype1, ntype2, expected_subtype",
+    [
+        ("longfloat", "longdouble", np.floating),
+        ("float16", "half", np.floating),
+        ("single", "float32", np.floating),
+        ("double", "float64", np.floating),
+        ("int8", "byte", np.integer),
+        ("short", "int16", np.integer),
+        ("intc", "int32", np.integer),
+        ("int0", "long", np.integer),
+        ("int", "long", np.integer),
+        ("int64", "longlong", np.integer),
+        ("int_", "intp", np.integer),
+        ("ubyte", "uint8", np.unsignedinteger),
+        ("uint16", "ushort", np.unsignedinteger),
+        ("uintc", "uint32", np.unsignedinteger),
+        ("uint", "uint64", np.unsignedinteger),
+        ("uintp", "ulonglong", np.unsignedinteger)
+    ]
+)
+def test_check_pandas_sparse_valid(ntype1, ntype2, expected_subtype):
+    # check that we support the conversion of sparse dataframe with mixed
+    # type which can be converted safely.
+    pd = pytest.importorskip("pandas", minversion="0.25.0")
+    df = pd.DataFrame({'col1': pd.arrays.SparseArray([0, 1, 0],
+                                                     dtype=ntype1),
+                       'col2': pd.arrays.SparseArray([1, 0, 1],
+                                                     dtype=ntype2)})
+    arr = check_array(df, accept_sparse=['csr', 'csc'])
+    assert np.issubdtype(arr.dtype, expected_subtype)
+
+
+@pytest.mark.parametrize("constructor_name", [
+    "list", "tuple", "array", "dataframe", "sparse_csr", "sparse_csc"
+])
+def test_num_features(constructor_name):
+    """Check _num_features for array-likes."""
+    X = [[1, 2, 3], [4, 5, 6]]
+    X = _convert_container(X, constructor_name)
+    assert _num_features(X) == 3
+
+
+@pytest.mark.parametrize(
+    "X",
+    [
+        [1, 2, 3],
+        ["a", "b", "c"],
+        [False, True, False],
+        [1.0, 3.4, 4.0],
+        [{"a": 1}, {"b": 2}, {"c": 3}],
+    ],
+    ids=["int", "str", "bool", "float", "dict"]
+)
+@pytest.mark.parametrize("constructor_name", [
+    "list", "tuple", "array", "series"
+])
+def test_num_features_errors_1d_containers(X, constructor_name):
+    X = _convert_container(X, constructor_name)
+    if constructor_name == "array":
+        expected_type_name = "numpy.ndarray"
+    elif constructor_name == "series":
+        expected_type_name = "pandas.core.series.Series"
+    else:
+        expected_type_name = constructor_name
+    message = (
+        "Unable to find the number of features from X of type "
+        f"{expected_type_name}"
+    )
+    if hasattr(X, "shape"):
+        message += " with shape (3,)"
+    elif isinstance(X[0], str):
+        message += " where the samples are of type str"
+    elif isinstance(X[0], dict):
+        message += " where the samples are of type dict"
+    with pytest.raises(TypeError, match=re.escape(message)):
+        _num_features(X)
+
+
+@pytest.mark.parametrize("X", [1, 'b', False, 3.0],
+                         ids=["int", "str", "bool", "float"])
+def test_num_features_errors_scalars(X):
+    msg = (
+        "Unable to find the number of features from X of type "
+        f"{type(X).__qualname__}"
+    )
+    with pytest.raises(TypeError, match=msg):
+        _num_features(X)
